@@ -1,11 +1,11 @@
+
 """Tasks are the main abstractions managed by doit"""
 
 import types
 import os
 import copy
-import sys
 
-from . import cmdparse
+from .cmdparse import CmdOption, TaskParse
 from .exceptions import CatchedException, InvalidTask
 from .action import create_action
 
@@ -38,17 +38,9 @@ class Task(object):
     @ivar doc: (string) task documentation
 
     @ivar options: (dict) calculated params values (from getargs and taskopt)
-    @ivar taskopt: (cmdparse.Command)
+    @ivar taskopt: (cmdparse.CmdParse)
     @ivar custom_title: function reference that takes a task object as
                         parameter and returns a string.
-    @ivar run_status (str): contains the result of Dependency.get_status
-            modified by runner, value can be:
-           - None: not processed yet
-           - run: task is selected to be executed (it might be running or
-                   waiting for setup)
-           - ignore: task wont be executed (user forced deselect)
-           - up-to-date: task wont be executed (no need)
-           - done: task finished its execution
     """
 
     DEFAULT_VERBOSITY = 1
@@ -59,7 +51,6 @@ class Task(object):
                   'actions': ((list, tuple,), (None,)),
                   'file_dep': ((list, tuple,), ()),
                   'task_dep': ((list, tuple,), ()),
-                  'result_dep': ((list, tuple,), ()),
                   'uptodate': ((list, tuple,), ()),
                   'calc_dep': ((list, tuple,), ()),
                   'targets': ((list, tuple,), ()),
@@ -75,13 +66,13 @@ class Task(object):
 
 
     def __init__(self, name, actions, file_dep=(), targets=(),
-                 task_dep=(), result_dep=(), uptodate=(),
+                 task_dep=(), uptodate=(),
                  calc_dep=(), setup=(), clean=(), teardown=(),
                  is_subtask=False, has_subtask=False,
                  doc=None, params=(), verbosity=None, title=None, getargs=None):
         """sanity checks and initialization
 
-        @param params: (list of option parameters) see cmdparse.Command.__init__
+        @param params: (list of dict for parameters) see cmdparse.CmdOption
         """
 
         getargs = getargs or {} #default
@@ -103,7 +94,7 @@ class Task(object):
         self.check_attr(name, 'title', title, self.valid_attr['title'])
 
         self.name = name
-        self.taskcmd = cmdparse.TaskOption(name, params, None, None)
+        self.taskcmd = TaskParse([CmdOption(opt) for opt in params])
         self.options = None
         self.setup_tasks = list(setup)
 
@@ -116,16 +107,6 @@ class Task(object):
 
         self._init_deps(file_dep, task_dep, calc_dep)
 
-        # DEPRECATED on 0.17 to be removed on 0.18
-        if result_dep: # pragma: no cover
-            from doit.tools import result_dep as check_result
-            uptodate = list(uptodate)
-            msg = ('DEPRECATION WARNING: Task "%s" is using "result_dep", '
-                   'use doit.tools.result_dep as "uptodate" instead.\n')
-            sys.__stderr__.write(msg % self.name)
-            for _result_dep in result_dep:
-                uptodate.append(check_result(_result_dep))
-
         self.value_savers = []
         self.uptodate = self._init_uptodate(uptodate) if uptodate else []
 
@@ -150,7 +131,6 @@ class Task(object):
 
         self.teardown = [create_action(a, self) for a in teardown]
         self.doc = self._init_doc(doc)
-        self.run_status = None
 
 
     def _init_deps(self, file_dep, task_dep, calc_dep):
@@ -255,32 +235,14 @@ class Task(object):
         self._init_options()
         for arg_name, desc in self.getargs.iteritems():
 
-            # DEPRECATED on 0.17, to be removed on 0.18
-            # value can be a string task_id.key_name
-            if isinstance(desc, basestring):
-                msg = ('DEPRECATION WARNING: Task "%s" "getargs" string values '
-                       '<task_id.key_name> is deprecated. '
-                       'Use a tuple (<task_id>, <key_name>) instead.\n')
-                sys.__stderr__.write(msg % self.name)
-
-                parts = desc.rsplit('.', 1)
-                if len(parts) != 2:
-                    msg = ("Taskid '%s' - Invalid format for getargs of '%s'.\n" %
-                           (self.name, arg_name) +
-                           "Should be <taskid>.<key-name> got '%s'\n" % desc)
-                    raise InvalidTask(msg)
-                self.getargs[arg_name] = parts
-            # END - DEPRECATION
-
-            # ... or tuple (task_id, key_name)
-            else:
-                parts = desc
-                if len(parts) != 2:
-                    msg = ("Taskid '%s' - Invalid format for getargs of '%s'.\n" %
-                           (self.name, arg_name) +
-                           "Should be tuple with 2 elements " +
-                           "('<taskid>', '<key-name>') got '%s'\n" % desc)
-                    raise InvalidTask(msg)
+            # tuple (task_id, key_name)
+            parts = desc
+            if isinstance(parts, basestring) or len(parts) != 2:
+                msg = ("Taskid '%s' - Invalid format for getargs of '%s'.\n" %
+                       (self.name, arg_name) +
+                       "Should be tuple with 2 elements " +
+                       "('<taskid>', '<key-name>') got '%s'\n" % desc)
+                raise InvalidTask(msg)
 
             if parts[0] not in self.setup_tasks:
                 self.setup_tasks.append(parts[0])
@@ -330,19 +292,6 @@ class Task(object):
         return self._action_instances
 
 
-    # deprecated on 0.17 to be removed on 0.18
-    def insert_action(self, call_ref):
-        """insert an action to execute before all other actions
-
-        @param call_ref: callable or (callable, args, kwargs)
-        This is part of interface to be used by 'uptodate' callables
-        """
-        msg = ('DEPRECATION WARNING: Task "%s" insert_action is deprecated, '
-               'use "value_savers" instead.\n')
-        sys.__stderr__.write(msg % self.name)
-        self.value_savers.append(call_ref)
-    # end deprecation
-
     def save_extra_values(self):
         """run value_savers updating self.values"""
         for value_saver in self.value_savers:
@@ -490,7 +439,6 @@ class Task(object):
         inst.wild_dep = self.wild_dep[:]
         inst.calc_dep = copy.copy(self.calc_dep)
         inst.doc = self.doc
-        inst.run_status = self.run_status
         return inst
 
     def __lt__(self, other):
