@@ -8,6 +8,7 @@ import operator
 import subprocess
 
 from . import exceptions
+from .dependency import UptodateCalculator
 from .action import CmdAction, PythonAction
 
 
@@ -31,6 +32,7 @@ def title_with_actions(task):
     return "%s => %s"% (task.name, title)
 
 
+
 # uptodate
 def run_once(task, values):
     """execute task just once
@@ -38,11 +40,55 @@ def run_once(task, values):
     """
     def save_executed():
         return {'run-once': True}
-    task.insert_action(save_executed)
+    task.value_savers.append(save_executed)
     return values.get('run-once', False)
 
 
 
+# uptodate
+class result_dep(UptodateCalculator):
+    """check if result of the given task was modified
+    """
+    def __init__(self, dep_task_name):
+        self.dep_name = dep_task_name
+        self.result_name = '_result:%s' % self.dep_name
+
+    def configure_task(self, task):
+        """to be called by doit when create the task"""
+        # result_dep creates an implicit task_dep
+        task.task_dep.append(self.dep_name)
+
+    def _result_single(self):
+        """get result from a single task"""
+        return self.get_val(self.dep_name, 'result:')
+
+    def _result_group(self, dep_task):
+        """get result from a group task
+        the result is the combination of results of all sub-tasks
+        """
+        prefix = dep_task.name + ":"
+        sub_tasks = {}
+        for sub in dep_task.task_dep:
+            if sub.startswith(prefix):
+                sub_tasks[sub] = self.get_val(sub, 'result:')
+        return sub_tasks
+
+    def __call__(self, task, values):
+        """return True if result is the same as last run"""
+        dep_task = self.tasks_dict[self.dep_name]
+        if not dep_task.has_subtask:
+            dep_result = self._result_single()
+        else:
+            dep_result = self._result_group(dep_task)
+        task.value_savers.append(lambda: {self.result_name: dep_result})
+
+        last_success = values.get(self.result_name)
+        if last_success is None:
+            return False
+        return (last_success == dep_result)
+
+
+
 # uptodate
 class config_changed(object):
     """check if passed config was modified
@@ -50,11 +96,11 @@ class config_changed(object):
     """
     def __init__(self, config):
         self.config = config
+        self.config_digest = None
 
-    def __call__(self, task, values):
-        config_digest = None
+    def _calc_digest(self):
         if isinstance(self.config, basestring):
-            config_digest = self.config
+            return self.config
         elif isinstance(self.config, dict):
             data = ''
             for key in sorted(self.config):
@@ -63,18 +109,22 @@ class config_changed(object):
                 byte_data = data.encode("utf-8")
             else:
                 byte_data = data
-            config_digest = hashlib.md5(byte_data).hexdigest()
+            return hashlib.md5(byte_data).hexdigest()
         else:
             raise Exception(('Invalid type of config_changed parameter got %s' +
                              ', must be string or dict') % (type(self.config),))
 
-        def _save_config():
-            return {'_config_changed': config_digest}
-        task.insert_action(_save_config)
+    def configure_task(self, task):
+        task.value_savers.append(lambda: {'_config_changed':self.config_digest})
+
+    def __call__(self, task, values):
+        """return True if confing values are UNCHANGED"""
+        self.config_digest = self._calc_digest()
         last_success = values.get('_config_changed')
         if last_success is None:
             return False
-        return (last_success == config_digest)
+        return (last_success == self.config_digest)
+
 
 
 # uptodate
@@ -100,7 +150,7 @@ class timeout(object):
     def __call__(self, task, values):
         def save_now():
             return {'success-time': time_module.time()}
-        task.insert_action(save_now)
+        task.value_savers.append(save_now)
         last_success = values.get('success-time', None)
         if last_success is None:
             return False
@@ -156,7 +206,7 @@ class check_timestamp_unchanged(object):
         """
         def save_now():
             return {self._key: self._get_time()}
-        task.insert_action(save_now)
+        task.value_savers.append(save_now)
 
         prev_time = values.get(self._key)
         if prev_time is None: # this is first run
