@@ -66,19 +66,8 @@ class TestTaskInit(object):
         t = task.Task("task5", ['action'], setup=["task2"])
         assert ["task2"] == t.setup_tasks
 
-    def test_run_status(self):
-        t = task.Task("t", ["q"])
-        assert t.run_status is None
-
 
 class TestTaskValueSavers(object):
-    def test_insert_action(self):
-        t = task.Task("Task X", ["taskcmd"])
-        def void(task, values, (my_arg1,)): pass
-        t.insert_action((void, [1]))
-        assert 1 == len(t.actions)
-        assert 1 == len(t.value_savers)
-
     def test_execute_value_savers(self):
         t = task.Task("Task X", ["taskcmd"])
         t.value_savers.append(lambda: {'v1':1})
@@ -188,12 +177,6 @@ class TestTask_Getargs(object):
         assert pytest.raises(task.InvalidTask, task.Task,
                               't3', None, getargs=getargs)
 
-    def test_many_dots(self):
-        # DEPRECATED
-        getargs = {'x': 't2:file.ext.x'}
-        t = task.Task('t1', None, getargs=getargs)
-        assert 't2:file.ext' in t.setup_tasks
-
 
 class TestTaskTitle(object):
 
@@ -293,24 +276,23 @@ class TestTaskTeardown(object):
 
 class TestTaskClean(object):
 
-    def pytest_funcarg__tmpdir(self, request):
-        def create_tmpdir():
-            tmpdir = {}
-            tmpdir['dir'] = tempfile.mkdtemp(prefix='doit-')
-            files = [os.path.join(tmpdir['dir'], fname)
-                     for fname in ['a.txt', 'b.txt']]
-            tmpdir['files'] = files
-            # create empty files
-            for filename in tmpdir['files']:
-                open(filename, 'a').close()
-            return tmpdir
-        def remove_tmpdir(tmpdir):
+    @pytest.fixture
+    def tmpdir(self, request):
+        tmpdir = {}
+        tmpdir['dir'] = tempfile.mkdtemp(prefix='doit-')
+        files = [os.path.join(tmpdir['dir'], fname)
+                 for fname in ['a.txt', 'b.txt']]
+        tmpdir['files'] = files
+        # create empty files
+        for filename in tmpdir['files']:
+            open(filename, 'a').close()
+
+        def remove_tmpdir():
             if os.path.exists(tmpdir['dir']):
                 shutil.rmtree(tmpdir['dir'])
-        return request.cached_setup(
-            setup=create_tmpdir,
-            teardown=remove_tmpdir,
-            scope="function")
+        request.addfinalizer(remove_tmpdir)
+
+        return tmpdir
 
 
     def test_clean_nothing(self, tmpdir):
